/**
 * @file
 * Implements the FolderShare folder browser menu user interface.
 *
 * The folder browser menu UI presents a menu button and pull-down menu that
 * lists commands that operate upon folders and files of various types.
 * Commands are plugins to the FolderShare module and implement operations
 * such as creating a new folder, uploading files, deleting, copying, moving,
 * and downloading. The server attaches command descriptions used here to
 * build commands and pre-validate them before submitting them to the server.
 *
 * Most commands use a selection, so this script supports selecting rows in
 * a folder browser. Rows can be selected individually or in groups.
 * Double-clicking a row opens the row's file or folder by advancing to its
 * page. Right-clicking on a row shows a context menu that shows a subset of
 * the main menu. Rows can be dragged and dropped onto subfolders to move
 * and copy, and files can be dragged from the host OS into the folder to
 * initiate a file upload.
 *
 * This script requires HTML elements added by a table view that uses a name
 * field formatter that attaches attributes to name field anchors. This script
 * uses those attributes to guide prevalidation of menu items as appropriate
 * for selected rows.
 *
 * This script also requires an HTML form that provides:
 * - A field to hold the current command choice.
 * - A set of fields holding command operands, including a parent ID,
 *   destination ID, and selection.
 * - A file field holding selected files for an upload.
 * - Drupal settings that list all known commands, and sundry other attributes.
 *
 * @ingroup foldershare
 * @see \Drupal\foldershare\Plugin\Field\FieldFormatter\FolderShareName
 * @see \Drupal\foldershare\Form\UIFolderBrowserMenu
 */
(function($, Drupal, drupalSettings) {

  Drupal.behaviors.foldershare_external_upload_functions = {
    attach(context) {
    },

    gather: function(env, $commandForm, utility) {
      const $s3CORSFileIdsInput = $(
          'input[name="foldershare-folder-browser-menu-s3-cors-file-ids"]',
          $commandForm)
          .eq(0);
      if ($s3CORSFileIdsInput.length === 0) {
        utility.printMalformedError("The main UI s3 CORS file ids input field is missing.");
        return false;
      }
      return {$s3CORSFileIdsInput: $s3CORSFileIdsInput};
    },

    executeUploadCommand: async function(env, ev) {
      const thisScript = Drupal.foldershare.UIFolderBrowserMenu;

      // Validate arguments.
      if (typeof env === "undefined") {
        return;
      }

      if (thisScript.uploadCommand !== "foldersharecommand_s3_cors_upload_files") {
        return;
      }

        // Clear any pending table refresh.
      if (env.refreshTimeout !== null) {
        clearTimeout(env.refreshTimeout);
        env.refreshTimeout = null;
      }

      const files = env.gather.$uploadInput[0].files;
      if (files.length === 0) {
        return;
      }

      // Get upload limits.
      const fileUploadSizeLimit =
          env.settings.foldershare.module.fileuploadsizelimit;
      const fileUploadNumberLimit =
          env.settings.foldershare.module.fileuploadnumberlimit;
      const postSizeLimit =
          env.settings.foldershare.module.postsizelimit;
      const fileExtensionsString = env.gather.$uploadInput.attr("accept");
      const fileExtensions = (fileExtensionsString === "") ? [] :
          fileExtensionsString.split(",");

      // Check number of files.
      if (files.length > fileUploadNumberLimit) {
        Drupal.foldershare.utility.showErrorDialog(
            Drupal.t("There are too many files to upload at once."),
            Drupal.t(
                "Please limit the number of files uploaded at once to @num.",
                { "@num": fileUploadNumberLimit }));
        env.gather.$commandForm[0].reset();
        return;
      }

      // Check individual file sizes and types. If there are problems,
      // there are two forms of the error message:
      // - Singular. Show the file name.
      // - Plural. Show the number of files, but no names.
      let errorFilename = "";
      let errorBadSizeNumber = 0;
      let errorBadExtensionNumber = 0;
      let errorUnreadableNumber = 0;
      let totalSize = 0;

      for (let i = 0; i < files.length; ++i) {
        // Get the file size, name, and derived extension (if any).
        const filesize = files[i].size;
        const filename = files[i].name;
        let fileext = "";
        if (filename.indexOf(".") !== (-1)) {
          fileext = "." + filename.split(".").pop().toLowerCase();
        }
        totalSize += filesize;

        // Verify that the file has a usable type and size, then check
        // if it is readable (and thus not a folder). If an error is
        // found, skip the rest of the checks.
        if (fileext !== "" && fileExtensions.length > 0 &&
            $.inArray(fileext, fileExtensions) === (-1)) {
          // It has an extension, and the extension is not allowed.
          ++errorBadExtensionNumber;
          errorFilename = filename;
        } else if (filesize > fileUploadSizeLimit) {
          // The size is too large.
          ++errorBadSizeNumber;
          errorFilename = filename;
        } else {
          try {
            await thisScript.validateFileIsReadable(files[i]);
          } catch (e) {
            // The file is not readable. It could be a folder.
            ++errorUnreadableNumber;
            errorFilename = filename;
          }
        }
      }

      // If there were any errors, show singular or plural messages.
      if (errorBadExtensionNumber > 0) {
        // One or more files have disallowed filename extensions. One or
        // more files might also have size issues, but those are secondary.
        // Report extensions only.
        const allExtensions = fileExtensions.join(", ");
        if (errorBadExtensionNumber === 1) {
          Drupal.foldershare.utility.showErrorDialog(
              Drupal.t(
                  "The file \"%filename\" cannot be uploaded.",
                  { "%filename": errorFilename }),
              Drupal.t(
                  "The file uses a file type that is not supported for uploads. The following types are supported: %list",
                  { "%list": allExtensions }));
        } else if (errorBadExtensionNumber === files.length) {
          Drupal.foldershare.utility.showErrorDialog(
              Drupal.t(
                  "None of the files can be uploaded."),
              Drupal.t(
                  "All of the files have types that are not supported for uploads. The following types are supported: %list",
                  { "%list": allExtensions }));
        } else {
          Drupal.foldershare.utility.showErrorDialog(
              Drupal.t(
                  "@count of the files cannot be uploaded.",
                  { "@count": errorBadExtensionNumber }),
              Drupal.t(
                  "Several files have types that are not supported for uploads. The following types are supported: %list",
                  { "%list": allExtensions }));
        }

        env.gather.$commandForm[0].reset();
        return;
      }

      if (errorBadSizeNumber > 0) {
        // One or more files have sizes that are too large.
        const withCommas = fileUploadSizeLimit.toString().replace(/\B(?=(\d{3})+(?!\d))/g, ",");

        if (errorBadSizeNumber === 1) {
          Drupal.foldershare.utility.showErrorDialog(
              Drupal.t(
                  "The file \"%filename\" cannot be uploaded.",
                  { "%filename": errorFilename }),
              Drupal.t(
                  "The file is too large to upload. Please limit files to @bytes bytes each.",
                  { "@bytes": withCommas }));
        } else if (errorBadSizeNumber === files.length) {
          Drupal.foldershare.utility.showErrorDialog(
              Drupal.t(
                  "None of the files can be uploaded."),
              Drupal.t(
                  "All of the files are too large to upload. Please limit files to @bytes bytes each.",
                  { "@bytes": withCommas }));
        } else {
          Drupal.foldershare.utility.showErrorDialog(
              Drupal.t(
                  "@count of the files cannot be uploaded.",
                  { "@count": errorBadSizeNumber }),
              Drupal.t(
                  "Several files are too large to upload. Please limit files to @bytes bytes each.",
                  { "@bytes": withCommas }));
        }

        env.gather.$commandForm[0].reset();
        return;
      }

      if (errorUnreadableNumber > 0) {
        // One or more files are not readable.
        if (errorUnreadableNumber === 1) {
          Drupal.foldershare.utility.showErrorDialog(
              Drupal.t(
                  "The file \"%filename\" cannot be uploaded.",
                  { "%filename": errorFilename }),
              Drupal.t(
                  "The file cannot be read. You may not have permission to read it, or it may be a folder."));
        } else if (errorUnreadableNumber === files.length) {
          Drupal.foldershare.utility.showErrorDialog(
              Drupal.t(
                  "None of the files can be uploaded."),
              Drupal.t(
                  "The files cannot be read. You may not have permission to read them, or they may be folders."));
        } else {
          Drupal.foldershare.utility.showErrorDialog(
              Drupal.t(
                  "@count of the files cannot be uploaded.",
                  { "@count": errorUnreadableNumber }),
              Drupal.t(
                  "The files cannot be read. You may not have permission to read them, or they may be folders."));
        }

        env.gather.$commandForm[0].reset();
        return;
      }

      // Verify total size of files is not too large.
      if (totalSize > postSizeLimit) {
        const withCommas = postSizeLimit.toString().replace(/\B(?=(\d{3})+(?!\d))/g, ",");
        Drupal.foldershare.utility.showErrorDialog(
            Drupal.t(
                "The total file upload size is too large."),
            Drupal.t(
                `The files together are too large to upload all at once. Please limit uploads to @bytes bytes.`,
                { "@bytes": withCommas }));
        env.gather.$commandForm[0].reset();
        return;
      }

      // Submit form.
      var target_id = ev.target.id;
      if (target_id.indexOf('--') >= 0) {
        target_id = target_id.split('--')[0];
      }
      var field_name = target_id.split('-');
      var field_name_array = field_name.slice(1, field_name.length - 2);
      var delta = field_name[field_name.length - 2];
      var field_name_key = field_name_array.join('_');

      var directory = env.settings.foldershare_s3_cors.upload_location.split('::')[1];

      // Encode any "/" chars in directory string
      directory = directory.replace(/\u002F/g, '::');

      var numTotalFiles = files.length;
      for (let i = 0; i < numTotalFiles; ++i) {
        var file_obj = files[i];

        // Check if file exists and get new key
        var ajax_uri = '/foldershare_s3_cors_ajax/s3_cors_key/' + encodeURIComponent(directory) + '/' + encodeURIComponent(file_obj.name);
        (function(file_obj, i) {
          $.get(ajax_uri, function (data, status) {

            if (status !== 'success') {
              Drupal.foldershare.utility.showErrorDialog(
                  Drupal.t(
                      "File upload failed."),
                  Drupal.t(
                      `An error occurred when uploading '@filename'.`,
                      { "@filename": file_obj.name }));
              return;
            }
            if (!data.file_key) {
              Drupal.foldershare.utility.showErrorDialog(
                  Drupal.t(
                      "File upload failed."),
                  Drupal.t(
                      `An uploaded file with the same name already exists.`));
            }

            var file_key = data.file_key;
            var file_name = data.file_name;
            var file_size = file_obj.size;

            // Use the HTML5 FormData API to build a POST form to send to S3.
            var fd = new FormData();
            fd.append('key', file_key);
            fd.append('Content-Type', file_obj.type);
            $.each(env.settings.foldershare_s3_cors.cors_form_data, function (key, value) {
              fd.append(key, value);
            });
            // File content must be the last item in the FormData.
            fd.append('file', file_obj);

            // Send the AJAX request to S3. Note: cors.form.action is the S3 URL
            // to POST this upload.
            $.ajax({
              url: env.settings.foldershare_s3_cors.cors_form_action,
              type: 'POST',
              enctype: 'multipart/form-data',
              data: fd,
              cache: false,
              contentType: false,
              processData: false,
              crossDomain: true,
              xhrFields: {
                withCredentials: true
              },

              error: function (jqXHR, textStatus, errorThrown, elem) {
                Drupal.foldershare.utility.showErrorDialog(
                    Drupal.t(
                        "File upload failed."),
                    Drupal.t(
                        `An error occurred when uploading '@filename'.`,
                        { "@filename": file_obj.name }));
              },

              success: function (data, textStatus, jqXHR) {
                // file_input.show();

                // Recover the file key. Note: The file_key contains directory as
                // well as file_name so encode any "/" chars in file_key string.
                var xmlDoc = $.parseXML(jqXHR.responseText);
                var $xml = $(xmlDoc);
                var file_key = $xml.find("Key").text();
                file_key = file_key.replace(/\u002F/g, '::');

                // The delta value is the sequence of upload sets to this field.
                var field_name = field_name_key + '_' + delta;

                // Save file in Drupal database
                var ajax_uri = '/foldershare_s3_cors_ajax/s3_cors_save/' + encodeURIComponent(file_key) + '/' + encodeURIComponent(file_name) + '/' + file_size + '/' + field_name;

                $.get(ajax_uri, function (data, status) {
                  if (!data.fid) {
                    if (data.errmsg) {
                      Drupal.foldershare.utility.showErrorDialog(
                          Drupal.t(
                              "File upload failed."),
                          Drupal.t(
                              data.errmsg));
                    }
                    else {
                      Drupal.foldershare.utility.showErrorDialog(
                          Drupal.t(
                              "File upload failed."),
                          Drupal.t(
                              `The file '@filename' couldn\'t  be saved on the site.`,
                              { "@filename": file_obj.name }));
                    }
                    return;
                  }

                  // Add the fid for this file to hidden fids field.
                  var fid = data.fid;

                  var fids = env.gather.$externalUploadInputs.$s3CORSFileIdsInput.val();
                  fids = (fids) ? fids + ' ' + fid : fid;
                  env.gather.$externalUploadInputs.$s3CORSFileIdsInput.val(fids);

                  // Post the results to Drupal if all files have been processed.
                  var num_fids = fids.split(' ').length;

                  if (num_fids === numTotalFiles) {
                    // The last file has been uploaded now.
                    // Remove the files from the file input so that they don't get sent to Drupal
                    env.gather.$uploadInput.val(null);

                    if (env.settings.foldershare.ajaxEnabled === true) {
                      env.gather.$commandSubmitButton.submit();
                    } else {
                      env.gather.$commandForm.submit();
                    }
                    // Clear the form so that it cannot be double-submitted.
                    env.gather.$commandForm[0].reset();
                  }
                });
              }
            });
          })
        })(file_obj, i);
      }
    },
  };

})(jQuery, Drupal, drupalSettings);
