<?php

namespace Drupal\foldershare_s3_cors\Controller;

use Drupal\Core\Controller\ControllerBase;
use Drupal\Core\Database\Connection;
use Drupal\Core\File\FileExists;
use Drupal\file\Entity\File;
use Drupal\foldershare_s3_cors\S3Settings;
use Drupal\s3fs\StreamWrapper\S3fsStream;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\HttpFoundation\JsonResponse;
use Symfony\Component\Mime\MimeTypeGuesserInterface;

/**
 * Default controller for the foldershare_s3_cors module.
 *
 * Provides controller functionality for foldershare S3 CORS AJAX operations.
 */
class S3CORSAjaxController extends ControllerBase {

  /**
   * S3 Settings.
   *
   * @var \Drupal\foldershare_s3_cors\S3Settings
   */
  protected $s3Config;

  /**
   * S3 Client Interface.
   *
   * @var \Aws\S3\S3ClientInterface
   */
  protected $s3Client;

  /**
   * Database connection.
   *
   * @var \Drupal\Core\Database\Connection
   */
  protected $database;

  /**
   * Mime Type Guesser Interface.
   *
   * @var \Drupal\Core\File\MimeType\MimeTypeGuesser
   */
  protected $mimeType;

  /**
   * AjaxController constructor.
   *
   * @param \Drupal\Core\Database\Connection $database
   *   The Drupal database connection service.
   * @param \Symfony\Component\Mime\MimeTypeGuesserInterface $mimeType
   *   The mime type guesser service.
   */
  public function __construct(Connection $database, MimeTypeGuesserInterface $mimeType) {
    $this->s3Config = S3Settings::getS3Settings();
    $this->s3Client = S3Settings::getAmazonS3Client($this->s3Config);
    $this->database = $database;
    $this->mimeType = $mimeType;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('database'),
      $container->get('file.mime_type.guesser'),
    );
  }

  /**
   * Return the file key (i.e. the path and name).
   */
  public function getKey($directory, $file_name, $replace = FileExists::Rename) {

    // Strip control characters (ASCII value < 32). Though these are allowed in
    // some filesystems, not many applications handle them well.
    $file_name = preg_replace('/[\x00-\x1F]/u', '_', $file_name);
    // Also replace forbidden chars if this is a Windows envrionment.
    if (substr(PHP_OS, 0, 3) == 'WIN') {
      // These characters are not allowed in Windows filenames.
      $file_name = str_replace([':', '*', '?', '"', '<', '>', '|'], '_', $file_name);
    }

    // Decode the "/" chars in the directory and build an initial file key.
    $directory = str_replace('::', '/', $directory);
    $file_key = $directory . '/' . $file_name;

    // Check if a file with this key already exists on S3.
    $file_exists = $this->s3FileExists($file_key);

    if ($file_exists) {
      switch ($replace) {
        case FileExists::Replace:
          // Do nothing here, we want to overwrite the existing file.
          break;

        case FileExists::Rename:
          $file_key = $this->createFileKey($directory, $file_name);
          break;

        case FileExists::Error:
          // Error reporting handled by calling function.
          return FALSE;
      }
    }
    // Core file_destination is not able to check remoe file existience.
    return new JsonResponse([
      'file_key' => $file_key,
      'file_name' => $file_name,
    ]);
  }

  /**
   * Create a new file key if the original one already exists.
   *
   * @throws \Exception
   */
  private function createFileKey($directory, $file_name) {

    /*
    $root_folder = '';
    $separator = '/';

    // A URI or path may already have a trailing slash or look like "public://".
    if (substr($directory, -1) == '/') {
    $separator = '';
    }
     */

    // Remove the root folder from the file directory if specified.
    $root_folder = '';
    // $config = $this->config('s3fs.settings');
    if (!empty($this->s3Config['root_folder'])) {
      $root_folder = $this->s3Config['root_folder'] . '/';
      $directory = str_replace($root_folder, '', $directory);
    }

    $separator = '/';
    // A URI or path may already have a trailing slash or look like "public://".
    if (substr($directory, -1) == '/') {
      $separator = '';
    }

    // Extract the file base name and the file extension (with leading period).
    $base_name = substr($file_name, 0, strrpos($file_name, '.'));
    $extension = substr($file_name, strrpos($file_name, '.'));

    $key_base = $root_folder . $directory . $separator . $base_name;

    if (S3Settings::getConfiguredS3Module() === "s3fs") {
      // Look in the s3fs cache to find files with a key like this.
      $uri_base = 's3://' . $directory . $separator . $base_name;
      $records = $this->database->select('s3fs_file', 's')
        ->fields('s', ['uri'])
        ->condition('uri', $this->database->escapeLike($uri_base) . '%', 'LIKE')
        ->execute()
        ->fetchCol();

      // Process the results array to extract the suffix values.
      $results = [];
      foreach ($records as $record) {
        $suffix = str_replace([$uri_base, $extension], '', $record);
        if ($suffix) {
          // Drop the leading underscore char.
          $suffix = (int) substr($suffix, 1);
          $results[$suffix] = $record;
        }
      }

      // Find a key suffix that can be used by
      // looking for a gap in suffix values.
      for ($suffix = 0; $suffix < count($results); $suffix++) {
        if (!isset($results[$suffix])) {
          break;
        }
      }
    }
    else {
      $suffix = 1;
    }

    // If we drop out the bottom then suffix will be one greater then largest
    // existing value.  Create a trial key and test.
    $trial_key = $key_base . '_' . $suffix . $extension;

    if ($this->s3FileExists($trial_key)) {
      // $suffix = max(array_keys($results));
      do {
        $trial_key = $key_base . '_' . ++$suffix . $extension;
      } while ($this->s3FileExists($trial_key));
    }

    return $trial_key;
  }

  /**
   * Check whehter a passed file name exists (using the file key).
   *
   * @throws \Exception
   */
  private function s3FileExists($key) {
    $bucket = $this->s3Config['bucket'] ?? '';
    return $this->s3Client->doesObjectExistV2($bucket, $key);
  }

  /**
   * Saves a file to S3 storage.
   *
   * @param string $file_path
   *   The path of the file to save.
   * @param string $file_name
   *   The name of the file to save.
   * @param int $file_size
   *   The size of the file in bytes.
   * @param string $field_name
   *   The field name associated with the file.
   *
   * @return \Symfony\Component\HttpFoundation\JsonResponse
   *   A JSON response containing the file information.
   */
  public function saveFile($file_path, $file_name, $file_size, $field_name) {
    $user = $this->currentUser();

    // Decode the "/" chars from file path.
    $file_path = str_replace('::', '/', $file_path);

    if (S3Settings::getConfiguredS3Module() === "s3fs") {
      // Remove the root folder from the file path if specified.
      $config = $this->config('s3fs.settings');
      if (!empty($config->get('root_folder'))) {
        $root_folder = $config->get('root_folder');
        $file_path = str_replace($root_folder . '/', '', $file_path);
      }
      /*
      // Remove the root folder from the file path if specified.
      if (!empty($this->s3Config['root_folder'])) {
        $root_folder = $this->s3Config['root_folder'];
        $file_path = str_replace($root_folder . '/', '', $file_path);
      }
      */
      $file_uri = 's3://' . $file_path;

      // Convert URLs back to their proper (original) file stream names.
      $public_folder = 's3://' . ($config->get('public_folder') ?: 's3fs-public');
      $private_folder = 's3://' . ($config->get('private_folder') ?: 's3fs-private');
      if (strpos($file_uri, $public_folder) === 0 || strpos($file_uri, $private_folder) === 0) {
        $file_uri = str_replace([$public_folder, $private_folder], ['public:/', 'private:/'], $file_uri);
      }

      // Record the uploaded file in the s3fs cache.
      // This needs to be done before
      // the file is saved so the the filesize can be found from the cache.
      // $wrapper = \Drupal::service("stream_wrapper.s3fs");
      // Record the uploaded file in the s3fs cache.
      // This needs to be done before
      // the file is saved so the filesize can be found from the cache.
      $wrapper = new S3fsStream();
      $wrapper->writeUriToCache($file_uri);

    }
    else {
      $file_uri = 's3://' . $file_path;
    }

    $file_mime = $this->mimeType->guessMimeType($file_name);

    $values = [
      'uid' => $user->id(),
      'uri' => $file_uri,
      'filename' => $file_name,
      'filemime' => $file_mime,
      'filesize' => $file_size,
      'status' => 0,
      'source' => $field_name,
    ];
    $file = File::create($values);

    $errors = [];
    $errors = array_merge($errors, $this->moduleHandler()->invokeAll('file_validate', [$file]));

    if (empty($errors)) {
      $file->save();
      $values['fid'] = $file->id();
      $values['uuid'] = $file->uuid();
    }
    else {
      $file->delete();
      $values['errmsg'] = implode("\n", $errors);
    }

    return new JsonResponse($values);
  }

}
