<?php

namespace Drupal\foldershare_s3_cors\Form;

use Aws\Exception\AwsException;

use Drupal\Core\Form\ConfigFormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\foldershare\Utilities\FileUtilities;
use Drupal\foldershare_s3_cors\S3Settings;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Config settings for foldershare_s3_cors.
 */
class SettingsForm extends ConfigFormBase {

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'foldershare_s3_cors_settings_form';
  }

  /**
   * {@inheritdoc}
   */
  protected function getEditableConfigNames() {
    return ['foldershare_s3_cors.settings'];
  }

  private function getS3ModuleOptions() {
      $moduleHandler = \Drupal::service('module_handler');

      $possible_s3_module_options = ['s3fs', 'flysystem_s3'];
      $s3_module_options = [];

      foreach($possible_s3_module_options as $s3_module) {
          if ($moduleHandler->moduleExists($s3_module)) {
              $s3_module_options[$s3_module] = $this->t($moduleHandler->getName($s3_module));
          }
      }
      return $s3_module_options;
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    $config = $this->configFactory->get('foldershare_s3_cors.settings');

    $s3StreamWrapper = FileUtilities::getStreamWrapper("s3");
    $s3StreamWrapperSupported = FALSE;
    if ($s3StreamWrapper) {
        $s3StreamWrapperSupported = FileUtilities::isStreamWrapperTypeSupported($s3StreamWrapper->getType());
    }

    $s3_module_settings = $this->getS3ModuleOptions();
    if (!$s3_module_settings) {
        $form['module_warning_message'] = [
            '#type' => 'markup',
            '#markup' => $this->t('<div>
                <h2>Warning: Could not detect a compatible S3 Drupal module</h2>
                <p>In order to enable S3 CORS uploads for FolderShare, you must install & enable at least one of the following Drupal S3 modules:</p>
                <ul>
                    <li>S3 File System (s3fs)</li>
                    <li>Flysystem Amazon S3 (flysystem_s3)</li>
                </ul>
            </div>')
        ];
    } else if (!$s3StreamWrapperSupported) {
        $form['module_warning_message'] = [
            '#type' => 'markup',
            '#markup' => $this->t("<div>
                <h2>Warning: Could not detect a valid <b><i>s3://</i></b> stream wrapper</h2>
                <p>In order to enable S3 CORS uploads for FolderShare, you must have a valid <b><i>s3://</i></b> stream wrapper installed in your Drupal site.</p>
            </div>")
        ];
    }
    else {
        $enable_s3_cors_uploads = $config->get('enable_s3_cors_uploads');
        $configured_s3_module = $config->get("configured_s3_module");

        // Commenting out the cors policy check below since not all S3-compatible APIs will implement the getBucketCors API.
        // For example, Minio allows CORS but intentionally returns a dummy error response for getBucketCors
        // See Minio example: https://github.com/minio/minio/blob/2bb6a3f4d076daca00c3a8c4fe29915c833cc9b3/cmd/dummy-handlers.go#L198

//        if ($enable_s3_cors_uploads && $configured_s3_module) {
//            $messenger = \Drupal::service("messenger");
//            try {
//                $s3Config = S3Settings::getS3Settings();
//                dpm($s3Config);
//                $s3Client = S3Settings::getAmazonS3Client($s3Config);
//
//                $bucketCorsResult = $s3Client->getBucketCors([
//                    'Bucket' => $s3Config['bucket'] ?? '', // REQUIRED
//                ]);
//                dpm("Bucket CORS:");
//                dpm($bucketCorsResult);
//            } catch (AwsException $e) {
//                $messenger->addError($this->t("Could not verify your S3 bucket's CORS policy due to the following error:"));
//                $messenger->addError($this->t($e->getMessage()));
//            } catch (\Exception $e) {
//                $messenger->addError($this->t("Could not verify your S3 bucket's CORS policy due to the following error:"));
//                $messenger->addError($this->t($e->getMessage()));
//            }
//        }

        $form['enable_s3_cors_uploads'] = [
            '#type' => 'checkbox',
            '#title' => $this->t('Enable S3 CORS uploads for FolderShare'),
            '#description' => $this->t("When enabled, this will send user-uploaded files directly to S3, without touching the site's web server."),
            '#default_value' => $enable_s3_cors_uploads,
        ];

        $form['configured_s3_module'] = [
            '#type' => 'select',
            '#title' => $this->t('Configured S3 module'),
            '#description' => $this->t("FolderShare will use the selected S3 module's settings & credentials to perform S3 CORS uploads."),
            '#default_value' => $configured_s3_module ?: FALSE,
            '#options' => $s3_module_settings,
            '#required' => $enable_s3_cors_uploads,
        ];

        $form = parent::buildForm($form, $form_state);
    }

    return $form;
  }

    public function validateForm(array &$form, FormStateInterface $form_state) {
        $enable_s3_cors_uploads = $form_state->getValue('enable_s3_cors_uploads');

        if ($enable_s3_cors_uploads) {
            $moduleHandler = \Drupal::service('module_handler');
            $configured_s3_module = $form_state->getValue('configured_s3_module');

            if (!$configured_s3_module || !$moduleHandler->moduleExists($configured_s3_module)) {
                $form_state->setErrorByName('configured_s3_module', $this->t('Please select an S3 module that is enabled.'));
            }

            $s3StreamWrapper = FileUtilities::getStreamWrapper("s3");
            $s3StreamWrapperSupported = FALSE;
            if ($s3StreamWrapper) {
                $s3StreamWrapperSupported = FileUtilities::isStreamWrapperTypeSupported($s3StreamWrapper->getType());
            }
            if (!$s3StreamWrapperSupported) {
                $form_state->setErrorByName('enable_s3_cors_uploads', $this->t('Please install an S3 stream wrapper for your Drupal site before enabling S3 CORS uploads for FolderShare.'));
            }
        }
    }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
      $enable_s3_cors_uploads = $form_state->getValue('enable_s3_cors_uploads');

      if ($enable_s3_cors_uploads) {
          foldershare_s3_cors_enable_cors_uploads();
      } else {
          foldershare_s3_cors_disable_cors_uploads();
      }

      $this->configFactory->getEditable('foldershare_s3_cors.settings')
          ->set('enable_s3_cors_uploads', $enable_s3_cors_uploads)
          ->set('configured_s3_module', $form_state->getValue('configured_s3_module'))
          ->save();

      parent::submitForm($form, $form_state);
  }

}
