<?php

namespace Drupal\foldershare_s3_cors\Plugin\FolderShareCommand;

//use Drupal;
//use Exception;
use Symfony\Component\DependencyInjection\ContainerInterface;

use Drupal\Core\Form\FormStateInterface;
use Drupal\foldershare\Plugin\FolderShareCommand\FolderShareCommandBase;

use Drupal\foldershare\Settings;
use Drupal\foldershare\Entity\FolderShare;

/**
 * Defines a command plugin to upload files for a folder.
 *
 * The command uploads files and adds them to the parent folder.
 *
 * Configuration parameters:
 * - 'parentId': the parent folder, if any.
 *
 * @ingroup foldershare
 *
 * @FolderShareCommand(
 *  id              = "foldersharecommand_s3_cors_upload_files",
 *  label           = @Translation("Upload (S3 CORS)"),
 *  menuNameDefault = @Translation("Upload (S3 CORS)..."),
 *  menuName        = @Translation("Upload (S3 CORS)..."),
 *  description     = @Translation("Upload files to this location (S3 CORS)."),
 *  category        = "import & export",
 *  weight          = 0,
 *  specialHandling = {
 *    "external_upload",
 *  },
 *  parentConstraints = {
 *    "kinds"   = {
 *      "rootlist",
 *      "folder",
 *    },
 *    "access"  = "create",
 *  },
 *  selectionConstraints = {
 *    "types"   = {
 *      "none",
 *    },
 *  },
 * )
 */
class S3CORSUploadFiles extends FolderShareCommandBase
{

    /*--------------------------------------------------------------------
     *
     * Construction.
     *
     *--------------------------------------------------------------------*/
    /**
     * Creates an instance of the plugin.
     *
     * @param ContainerInterface $container
     *   The container from which to get services.
     * @param array $configuration
     *   The plugin configuration.
     * @param string $pluginId
     *   The plugin ID.
     * @param mixed $pluginDefinition
     *   The plugin implementation definition.
     */
    public static function create(
        ContainerInterface $container,
        array $configuration,
        $plugin_id,
        $plugin_definition)
    {
        return new static(
            $container,
            $configuration,
            $plugin_id,
            $plugin_definition);
    }

    /*--------------------------------------------------------------------
     *
     * Configuration form.
     *
     *--------------------------------------------------------------------*/
    /**
     * {@inheritdoc}
     */
    public function hasConfigurationForm()
    {
        return FALSE;
    }

    /**
     * {@inheritdoc}
     */
    public function buildConfigurationForm(
        array $form,
        FormStateInterface $formState)
    {
        return $form;
    }

    /**
     * {@inheritdoc}
     */
    public function submitConfigurationForm(
        array &$form,
        FormStateInterface $formState)
    {
        $this->execute();
    }

    /*--------------------------------------------------------------------
     *
     * Execute.
     *
     *--------------------------------------------------------------------*/
    /**
     * {@inheritdoc}
     */
    public function execute()
    {
        // Get the parent, if any.
        $parent = $this->getParent();

        $configuration = $this->getConfiguration();
        $s3CORSFileIds = $configuration['foldershare_s3_cors_file_ids'];

        // Attach the uploaded files to the parent. The files have been uploaded to S3 already.
        // File objects have also been created for them.
        // It remains to wrap them with FolderShare entities and add them to the parent folder.

        try {
            if ($parent === NULL) {
                $results = FolderShare::addFileIdsToRootFromFormUpload(
                    $s3CORSFileIds,
                    FALSE,
                    (-1));
            } else {
                $results = $parent->addFileIdsFromFormUpload(
                    $s3CORSFileIds,
                    FALSE,
                    (-1));
            }
        } catch (\Exception $e) {
            $this->getMessenger()->addError($e->getMessage());
            return;
        }

        // The returned array has one entry for each uploaded file. Each entry
        // is an associative array containing:
        // - 'filename': The client original filename for the file.
        // - 'error': The error message, if any.
        // - 'foldershare': The created FolderShare entity, if any.
        $nUploaded = 0;
        $firstItem = NULL;
        foreach ($results as $entry) {
            if (isset($entry['foldershare']) === FALSE) {
                // This file had an upload error.
                if (isset($entry['error']) === TRUE) {
                    $this->getMessenger()->addError($entry['error']);
                } else {
                    $filename = $entry['filename'];
                    $this->getMessenger()->addError(
                        "An unknown error occurred for $filename.");
                }
            } else {
                if ($nUploaded === 0) {
                    $firstItem = $entry;
                }

                $nUploaded++;
            }
        }

        if (Settings::getCommandNormalCompletionReportEnable() === TRUE) {
            // Report on results, if any.
            if ($nUploaded === 1) {
                // Single file. There may have been other files that had errors though.
                $name = $firstItem->getFilename();
                $this->getMessenger()->addStatus(
                    $this->t(
                        "The @kind '@name' has been uploaded.",
                        [
                            '@kind' => FolderShare::translateKind(FolderShare::FILE_KIND),
                            '@name' => $name,
                        ]));
            } elseif ($nUploaded > 1) {
                // Multiple files. There may have been other files that had errors tho.
                $this->getMessenger()->addStatus(
                    $this->t(
                        "@number @kinds have been uploaded.",
                        [
                            '@number' => $nUploaded,
                            '@kinds' => FolderShare::translateKinds(FolderShare::FILE_KIND),
                        ]));
            }
        }

    }

}
