<?php

namespace Drupal\foldershare_s3_cors;

use Aws\Credentials\CredentialProvider;
use Aws\S3\S3Client;
use Drupal\Component\Utility\UrlHelper;
use Drupal\Core\Site\Settings as SiteSettings;

/**
 *
 */
class S3Settings {

  /**
   *
   */
  public static function getConfiguredS3Module() {
    $foldershare_s3_cors_config = \Drupal::config('foldershare_s3_cors.settings');
    return $foldershare_s3_cors_config->get("configured_s3_module");
  }

  /**
   *
   */
  public static function isConfiguredS3ModuleValid($configured_s3_module) {
    return in_array($configured_s3_module, ["s3fs", "flysystem_s3"]);
  }

  /**
   *
   */
  public static function getS3Settings() {
    $foldershare_s3_cors_config = \Drupal::config('foldershare_s3_cors.settings');
    $configured_s3_module = $foldershare_s3_cors_config->get("configured_s3_module");
    if (!S3Settings::isConfiguredS3ModuleValid($configured_s3_module)) {
      return [];
    }

    if ($configured_s3_module === "s3fs") {
      // S3fs.
      $moduleHandler = \Drupal::service('module_handler');

      $s3fs_config = \Drupal::service('config.factory')->getEditable('s3fs.settings');
      // dpm($s3fs_config);.
      $bucket = $s3fs_config->get('bucket');

      $region = $s3fs_config->get('region') ?: SiteSettings::get('s3fs.region', '');
      $s3fs_config->set("region", $region);

      $access_key = $s3fs_config->get('access_key') ?: SiteSettings::get('s3fs.access_key', '');
      $secret_key = $s3fs_config->get('secret_key') ?: SiteSettings::get('s3fs.secret_key', '');

      $noKeyInSettings = (empty($access_key) || empty($secret_key));

      if ($noKeyInSettings && $moduleHandler->moduleExists('key')) {
        if (!$access_key && !empty($s3fs_config->get('keymodule')['access_key_name'])) {
          $key = \Drupal::service('key.repository')->getKey($s3fs_config->get('keymodule')['access_key_name']);
          $key_value = $key->getKeyValue();
          if (!empty($key_value)) {
            $access_key = $key_value;
          }
        }

        if (!$secret_key && !empty($s3fs_config->get('keymodule')['secret_key_name'])) {
          $key = \Drupal::service('key.repository')->getKey($s3fs_config->get('keymodule')['secret_key_name']);
          $key_value = $key->getKeyValue();
          if (!empty($key_value)) {
            $secret_key = $key_value;
          }
        }
      }

      if (!empty($access_key) && !empty($secret_key)) {
        $s3fs_config->set('credentials', [
          'key' => $access_key,
          'secret' => $secret_key,
        ]);
      }
      else {
        // Use the defaultProvider to get all paths in home, env, etc.
        $provider = CredentialProvider::defaultProvider();

        // Use a custom ini file before defaultProvider.
        if (!empty($s3fs_config->get('credentials_file', ''))) {
          $provider = CredentialProvider::chain(
                CredentialProvider::ini(NULL, $s3fs_config->get('credentials_file', '')),
                $provider);
        }

        // Cache the results in a memoize function to avoid loading and parsing
        // the ini file on every API operation.
        $provider = CredentialProvider::memoize($provider);
        $s3fs_config->set('credentials', $provider);
      }

      $s3fs_config->set("access_key", $access_key);
      $s3fs_config->set("secret_key", $secret_key);

      $hostname = $s3fs_config->get('use_customhost') ? $s3fs_config->get('hostname') : 's3.' . $region . '.amazonaws.com';
      $bucket_endpoint = $s3fs_config->get('use_path_style_endpoint') ? $hostname . '/' . $bucket : $bucket . '.' . $hostname;

      if (!empty($hostname)) {
        if (preg_match('#http(s)?://#i', $hostname) === 1) {
          $s3fs_config->set('endpoint', $hostname);
        }
        else {
          // Fallback for before we required hostname to include protocol.
          $defaultProtocol = \Drupal::service('request_stack')
            ->getCurrentRequest()
            ->getScheme();

          $s3fs_config->set('endpoint', ($s3fs_config->get('use_https') ? 'https://' : "$defaultProtocol://") . $hostname);
        }
      }

      if (!empty($bucket_endpoint)) {
        if (preg_match('#http(s)?://#i', $bucket_endpoint) !== 1) {
          // Protocol isn't included in the bucket endpoint.
          $defaultProtocol = \Drupal::service('request_stack')
            ->getCurrentRequest()
            ->getScheme();

          $bucket_endpoint = ($s3fs_config->get('use_https') ? 'https' : $defaultProtocol) . '://' . $bucket_endpoint . '/';
        }
      }

      $s3fs_config->set("s3_cors_form_action", $bucket_endpoint);
      return $s3fs_config->get();
    }
    else {
      // Flysystem S3.
      $flysystem_s3_config = SiteSettings::get("flysystem");
      $flysystem_s3_config = $flysystem_s3_config['s3']['config'];

      $flysystem_s3_config['access_key'] = $flysystem_s3_config['key'];
      $flysystem_s3_config['secret_key'] = $flysystem_s3_config['secret'];

      $access_key = $flysystem_s3_config['access_key'];
      $secret_key = $flysystem_s3_config['secret_key'];

      if (!empty($access_key) && !empty($secret_key)) {
        $flysystem_s3_config['credentials'] = [
          'key' => $access_key,
          'secret' => $secret_key,
        ];
      }
      else {
        // Use the defaultProvider to get all paths in home, env, etc.
        $provider = CredentialProvider::defaultProvider();

        // Cache the results in a memoize function to avoid loading and parsing
        // the ini file on every API operation.
        $provider = CredentialProvider::memoize($provider);
        $flysystem_s3_config['credentials'] = $provider;
      }

      // Add keys: disable_cert_verify (optional), disable_shared_config_files (optional), s3_cors_form_action (required)
      $flysystem_s3_config['s3_cors_form_action'] = S3Settings::calculateBucketUrlPrefixForFlysystemS3($flysystem_s3_config);
      return $flysystem_s3_config;
    }
  }

  /**
   * The function below is adapted from the flysystem_s3 module's implementation.
   */
  private static function calculateBucketUrlPrefixForFlysystemS3(array $config) {
    $defaultProtocol = \Drupal::service('request_stack')
      ->getCurrentRequest()
      ->getScheme();
    $protocol = $config['protocol'] ?? $defaultProtocol;

    $cname = (string) $config['cname'] ?? '';

    $prefix = '';

    if ($cname !== '' && $config['cname_is_bucket'] ?? TRUE) {
      return $protocol . '://' . $cname . $prefix;
    }

    $bucket = (string) $config['bucket'] ?? '';
    $bucket = $bucket === '' ? '' : '/' . UrlHelper::encodePath($bucket);

    // No custom CNAME was provided. Generate the default S3 one.
    if ($cname === '') {
      $cname = 's3-' . $config['region'] ?? 'us-east-1' . '.amazonaws.com';
    }

    // us-east-1 doesn't follow the consistent mapping.
    if ($cname === 's3-us-east-1.amazonaws.com') {
      $cname = 's3.amazonaws.com';
    }

    return $protocol . '://' . $cname . $bucket . $prefix;
  }

  /**
   *
   */
  public static function getAmazonS3Client(array $config) {
    $s3 = &drupal_static(__METHOD__ . '_S3Client');
    $static_config = &drupal_static(__METHOD__ . '_static_config');

    // If the client hasn't been set up yet, or the config given to this call is
    // different from the previous call, (re)build the client.
    if (!isset($s3) || $static_config != $config) {
      $client_config = [];
      $client_config['credentials'] = $config['credentials'];

      if (!empty($config['region'] ?? '')) {
        $client_config['region'] = $config['region'];
        // Signature v4 is only required in the Beijing and Frankfurt regions.
        // Also, setting it will throw an exception if a region hasn't been set.
        $client_config['signature'] = 'v4';
      }

      if (!empty($config['endpoint'] ?? '')) {
        $client_config['endpoint'] = $config['endpoint'];
      }

      // Use path-style endpoint, if selected.
      if (!empty($config['use_path_style_endpoint'])) {
        $client_config['use_path_style_endpoint'] = TRUE;
      }

      $client_config['version'] = Constants::S3_API_VERSION;

      // Disable SSL/TLS verification.
      if (!empty($config['disable_cert_verify'] ?? '')) {
        $client_config['http']['verify'] = FALSE;
      }
      // Set use_aws_shared_config_files = false to avoid reading ~/.aws/config.
      // If open_basedir restriction is in effect an exception may be thrown
      // without this enabled.
      if (!empty($config['disable_shared_config_files'] ?? '')) {
        $client_config['use_aws_shared_config_files'] = FALSE;
      }
      // Create the Aws\S3\S3Client object.
      $s3 = new S3Client($client_config);
      $static_config = $config;
    }
    return $s3;
  }

}
